"""
             Yasuharu Okamoto, 2021.11, 12
"""
import numpy as np
import scipy.stats as ss
import scipy.integrate as si
import matplotlib.pyplot as plt
import seaborn as sb

N = int(input('N = '))
k = int(input('k = '))

class Triangular:
    def __init__(self, d_prime):
        self.d_prime = d_prime
        
    def func(self, u):
        v = ss.norm.cdf(-u * (3.0**0.5) + self.d_prime * ((2/3)**0.5)) + \
                ss.norm.cdf(-u * (3.0**0.5) - self.d_prime * ((2/3)**0.5)) 
        return v * np.exp(-0.5 * (u**2)) / ((2.0 * np.pi)**0.5)

#
#      Probability of the correct response for d' = d_prime 
#
def Pc_d_prime(d_prime):
    d_func = Triangular(d_prime).func
    return 2 * si.quad(d_func, 0.0, +np.inf)[0]

#
#       Probabilities of correct responses at the grid points
#       Grid is constructed on interval [0, 10] with 1001 points
#
dprimes = np.linspace(0, 10, 1001)    
for i in range(len(dprimes)):
    if i % 100 == 0:
        print('{0:}/{1:}, calculating...     '.format(i, len(dprimes)))  #, end = '\r')
    dprimes[i] = Pc_d_prime(i/100)

#
#       Posterior distribution of d'
#
postPdprime = np.empty(len(dprimes))
for i in range(len(dprimes)):
    postPdprime[i] = (dprimes[i] ** k) * ((1 - dprimes[i]) ** (N - k))
postPdprime = (postPdprime / np.sum(postPdprime))

#
#        MAP estimate
#
map_idx = np.argmax(postPdprime)  
MAP_Est = map_idx / 100       
print('MAP_Est =', MAP_Est)

#
#      Cumuloative distribution of the posterior distribution of d'
#      HDI is caluculated using this cumulative distribution
#
postPcum = np.cumsum(postPdprime)

#
#      Check HDI from the left
#
a = 0.05
Lp = 0
Rp = 0  
w_t = 1000
L_t = Lp
R_t = Rp
while True:
    if Lp == 0:
        cumPL = 0.0
    else:
        cumPL = postPcum[Lp-1]
    while 1.0 - postPcum[Rp] >= a - cumPL:
        Rp += 1
    if Lp == 0:
        L_t = Lp
        R_t = Rp
        w_t = R_t - L_t        
    elif Rp - Lp < w_t:
        L_t = Lp
        R_t = Rp
        w_t = R_t - L_t
    Lp += 1
    if postPcum[Lp] >= a:
        Lp -= 1
        break

#
#     Check HDI from the right
#
Lp = 1000
Rp = 1000
w_t = 1000
L_t1 = Lp
R_t1 = Rp
while True:
    if Rp == 1000:
        cumPR = 0.0
    else:
        cumPR = 1.0 - postPcum[Rp]
    while postPcum[Lp] >= a - cumPR:
        if Lp > 0:
            Lp -= 1
        else:
            break
    if Rp == 1000:
        L_t1 = Lp
        R_t1 = Rp
        w_t = R_t1 - L_t1
    elif Rp - Lp < w_t:
        L_t1 = Lp
        R_t1 = Rp
        w_t = R_t1 - L_t1
    Rp -= 1
    if 1.0 - postPcum[Rp] >= a:
        Rp += 1
        break

#
#       Calculate the means of the left and right checks above
#
HDI_L = ((L_t + L_t1) / 2.0) / 100.0
HDI_R = ((R_t + R_t1) / 2.0) / 100.0

plt.plot(np.linspace(0, 10, 1001), postPdprime)
plt.plot([MAP_Est,MAP_Est], [0, postPdprime[map_idx]], label = 'Mode = {}'.format(MAP_Est))
plt.title(f"Psterior Distribution for d'\nN = {N}, k = {k}", fontsize = 14)
plt.xlabel("d'", fontsize = 12)
plt.plot([0, 10], [0, 0], lw = 1, c = 'k')
plt.yticks([0])
plt.plot([HDI_L, HDI_R], [0, 0], lw = 5, label = '95% HDI = [{0:.2f}, {1:.2f}]'.format(HDI_L, HDI_R))
plt.legend()
plt.savefig(f'FigN{N}k{k}.png')
plt.show()
